-- CAUTION: Without appropriate time delays, CPU usage will approach 100%
-- Library is alpha 0.1
-- objective: transfer data between main & thread:
--	options: 'ready', 'standby' or 'received'
--	main can send or receive
--	thread can send or receive

local Communicate	= {}
Communicate.main	= {}
Communicate.thread	= {}

function Communicate.createMainThread()
	thread			= love.thread.newThread( 'thread', 'thread.lua' )
	thread:start()
	thread:set( 'main_send', 'standby' )
	thread:set( 'main_receive', 'standby' )
	end

function Communicate.createBackgroundThread()
	bkgnd	= love.thread.getThread()
	bkgnd:set( 'thread', 'running' )
	bkgnd:set( 'thread_receive','ready' )
	end

function Communicate.main.sendData( data )
	if thread:peek(  'thread_receive' ) == 'ready' and	-- thread wants data
		thread:peek( 'main_send' ) ~= 'ready' then		-- data not available
		thread:set(  'mainData', data )					-- make data available
		thread:set(  'main_send', 'ready' )
		else
		if thread:peek( 'thread_receive' ) == 'received' then -- data transferred
			thread:set( 'main_send', 'standby' )
			thread:set( 'main_receive', 'ready' )
			return true
			end
		end
	return false		-- transfer didn't occur
	end

function Communicate.main.getData()		-- receive data from thread (if available)
	if thread:peek( 'thread_send') == 'ready' then
		local data = thread:peek( 'threadData' )
		thread:set( 'main_receive', 'received' )
		thread:set( 'thread_receive', 'ready' )
		return data		-- data available
		else
		return false	-- data not available
		end
	end

function Communicate.thread.getData()		-- receive data from thread (if available)
	if bkgnd:peek( 'main_send' ) == 'ready' then
		local data = bkgnd:peek('mainData')
		bkgnd:set( 'thread_receive', 'received' )
		return data		-- data available
		else
		return false	-- data not available
		end
	end

function Communicate.thread.sendData( data )
	if bkgnd:peek(  'main_receive' ) == 'ready' and	-- thread wants data
		bkgnd:peek( 'thread_send' ) ~= 'ready' then		-- data not available
		bkgnd:set(  'threadData', data )					-- make data available
		bkgnd:set(  'thread_send', 'ready' )
		else
		if bkgnd:peek( 'main_receive' ) == 'received' then -- data transferred
			bkgnd:set( 'thread_send', 'standby' )
			bkgnd:set( 'thread_receive', 'ready' )
			bkgnd:set('main_receive','standby')
			return true
			end
		return false	-- transfer didn't occur
		end
	end

function Communicate.show_status()
	------------------------
	-- display thread status
	------------------------
	local thread_messages = thread:getKeys()
	local status_x, status_y = 10, 7	-- display location
	if thread:peek( 'thread' )=='running'  then
		gr.setColor( 200, 0, 55, 255 )		-- red border
		gr.print( "Thread Traffic", status_x, status_y )
		gr.rectangle( 'line', 5, 5, 175, status_y + #thread_messages * ( fontsize )+fontsize )
		gr.rectangle( 'line', 5, 5, 175, fontsize+4)
		gr.setColor( 185, 185, 0, 255 )	-- yellow text
		for i = 1, #thread_messages do
			local msg = tostring( thread:peek( thread_messages[i] ) )
			gr.print( thread_messages[i]..": "..msg, status_x, status_y + (i) * fontsize )
			end
		end
	end

return Communicate
