
-- The Ref3D Library
-- License : Depends on what substitute541 is using today
-- This is an unauthorized (personal) version of substitute541's library

-- Settings
local Ref3D		= {}
Ref3D.graphics	= {}
Ref3D.func		= {}
Ref3D.light		= {}
Ref3D.version	= '0.3.2.mod+'
pi, sqrt, acos, min = math.pi, math.sqrt, math.acos, math.min

function Ref3D.load( fl, vpX, vpY )
	focalLength			= fl
	vanishingPointX, vanishingPointY = vpX, vpY
	zSortUsingMinZ		= ffalse
	cullBackfacing = true
	end

function Ref3D.convert3Dto2D( pt )	-- 3D => 2D
	local scale	= focalLength/(focalLength + pt[3])
	local newX	= vanishingPointX + pt[1] * scale
	local newY	= vanishingPointY + pt[2] * scale
	return { newX, newY }
	end

-- Check for backface
function Ref3D.isBackFace( poly )	-- left handed triangles not shown
	local cx = poly[3][1] - poly[1][1]
	local cy = poly[3][2] - poly[1][2]
	local bx = poly[2][1] - poly[3][1]
	local by = poly[2][2] - poly[3][2]
	return cx * by >= cy * bx	-- <= change (equal added)- triangle perpendicular to screen
	end

-- Draws a 2D Circle	-- for light source
function Ref3D.graphics.circle2D( mode, lgt, radius, segments )
	local scale	= focalLength / ( focalLength + lgt[3] )
	local New	= Ref3D.convert3Dto2D( lgt )
	love.graphics.circle( mode, New[1], New[2], radius * scale, segments )
	end

-- Draws a triangle
function Ref3D.graphics.triangle( mode, bfc, tri, ox, oy, sz )
	local tri = {Ref3D.convert3Dto2D( tri[1] ),Ref3D.convert3Dto2D( tri[2] ),Ref3D.convert3Dto2D( tri[3] )}
	if bfc then											-- back facing culling
		if Ref3D.isBackFace( tri ) then
			return										-- (don't draw back facing)
			end
		end
	love.graphics.triangle(mode,
		sz * ( tri[1][1] - vp.x ) +vp.x + ox, sz * ( tri[1][2] - vp.y ) + vp.y + oy,
		sz * ( tri[2][1] - vp.x ) +vp.x + ox, sz * ( tri[2][2] - vp.y ) + vp.y + oy,
		sz * ( tri[3][1] - vp.x ) +vp.x + ox, sz * ( tri[3][2] - vp.y ) + vp.y + oy)
	end

-- Rotates point around vanishing points
function Ref3D.func.rotateX(y, z, angleX, centerY, centerZ)
	local y1 = (y - centerY) * cos(angleX) - (z - centerZ) * sin(angleX)
	local z1 = (z - centerZ) * cos(angleX) + (y - centerY) * sin(angleX)
	return y1 + centerY, z1 + centerZ
	end

function Ref3D.func.rotateY(x, z, angleY, centerX, centerZ)
	local x1 = (x - centerX) * cos(angleY) - (z - centerZ) * sin(angleY)
	local z1 = (z - centerZ) * cos(angleY) + (x - centerX) * sin(angleY)
	return x1 + centerX, z1 + centerZ
	end

function Ref3D.func.rotateZ(x, y, angleZ, centerX, centerY)
	local x1 = (x - centerX) * cos(angleZ) - (y - centerY) * sin(angleZ)
	local y1 = (y - centerY) * cos(angleZ) + (x - centerX) * sin(angleZ)
	return x1 + centerX, y1 + centerY
	end

-- Finds the average Z
function Ref3D.func.getAvgZTriangle(az, bz, cz)
	local avgZ = (az + bz + cz)/3
	return avgZ
	end

-- Sorts by average z
function Ref3D.func.zSortTriangle( poly, pts )	-- if zSortUsingMinZ = false
	for i = 1, #poly do
		poly[i].avgZ = Ref3D.func.getAvgZTriangle(pts[poly[i][1]][3], pts[poly[i][2]][3], pts[poly[i][3]][3])
		end
	table.sort(poly, function(A, B) return A.avgZ > B.avgZ end)
	end
-- Sorts by minimum z
function Ref3D.func.zSortTriangle2( poly, pts )	-- if zSortUsingMinZ = true
	for _, p in ipairs(poly) do
		p.minZ = math.min(pts[p[1]][3], pts[p[2]][3], pts[p[3]][3])
		end
	table.sort(poly, function(A, B) return A.minZ > B.minZ end)
	end

function Ref3D.loadData( name )
	--file structure
	-- POINTS
	-- x1,y1,z1
	-- x2,y2,z2
	-- ...
	-- POLY
	-- t1,t2,t3
	-- t2,t3,t4
	-- ...
	local points2, polygons2 = {},{}
	local mode = 'none'
	for line in love.filesystem.lines( name ) do
		if line == 'POINTS' then mode = 1
			elseif line == 'POLY' then mode = 2
			else
				if mode ==1 then table.insert(points2,line:split())
					elseif mode == 2 then table.insert(polygons2,line:split())
					end
			end
		end
	for i = 1, #polygons2 do
		polygons2[i][1] = tonumber(polygons2[i][1])
		polygons2[i][2] = tonumber(polygons2[i][2])
		polygons2[i][3] = tonumber(polygons2[i][3])
		polygons2[i].color = {}
		end
	return points2, polygons2
	end

function Ref3D.sight()		-- just an sight image
	if zoom < 1.5 then gr.setColor(0,255,0,255) elseif
		zoom <2 then gr.setColor(255,255,0,255)
		else gr.setColor(255,0,0,255) end
	r,cx,cy= 20*zoom,screenWidth/2,screenHeight/2
	gr.circle('line',cx,cy,r,16)	-- center circle
	gr.line(cx-2*r,cy,cx-r,cy)	-- cross pattern
	gr.line(cx+2*r,cy,cx+r,cy)
	gr.line(cx,cy-2*r,cx,cy-r)
	gr.line(cx,cy+2*r,cx,cy+r)

	gr.line(cx-4*r,cy-3*r,cx-3*r,cy-3*r)	-- corner angles
	gr.line(cx-4*r,cy-3*r,cx-4*r,cy-2*r)
	gr.line(cx+4*r,cy-3*r,cx+3*r,cy-3*r)
	gr.line(cx+4*r,cy-3*r,cx+4*r,cy-2*r)
	gr.line(cx-4*r,cy+3*r,cx-3*r,cy+3*r)
	gr.line(cx-4*r,cy+3*r,cx-4*r,cy+2*r)
	gr.line(cx+4*r,cy+3*r,cx+3*r,cy+3*r)
	gr.line(cx+4*r,cy+3*r,cx+4*r,cy+2*r)
	end

function Ref3D.extractPoints( stringData )	 -- string =>  point table{x,y,z}
	local temp 			= stringData:split() -- p1x,p1y,p1z,... => temp{p1x,p1y,p1z, ... }
	local reconstructedTable	= {}
	local field			= 1
	for i = 1, #temp, 3 do		-- temp{p1x,p1y,p1z, ... } => table{{p1x,p1y,p1z},{p2x,p2y,p2z}, ... }
		reconstructedTable[ field ] = { temp[i], temp[i+1], temp[i+2] }
		field = field + 1
		end
	return reconstructedTable
	end

function string:split( sep ) -- parces string 'x,y,z' into x y z - default seoarator == comma
		local sep, fields	= sep or ",", {}
		local pattern		= string.format( "([^%s]+)", sep )
		self:gsub(pattern, function(c) fields[#fields+1] = c end)
		return fields
		end

function Ref3D.light.load()
	light						= {0,0,0}
	light.brightness			= 0
	light.r, light.g, light.b	= 255, 255, 255
	light.newColor				= {}
	light.factor				= 0
	light.ar, light.ag, light.ab = 0, 0, 0
	end

function Ref3D.light.setAmbience(r, g, b)
	light.ar, light.ag, light.ab = r, g, b
	end

function Ref3D.light.getLightFactor( poly )
	local pointA, pointB, pointC	= points[poly[1]],points[poly[2]],points[poly[3]]
	local ab	= { pointA[1] - pointB[1], pointA[2] - pointB[2], pointA[3] - pointB[3]}
	local bc	= { pointB[1] - pointC[1], pointB[2] - pointC[2], pointB[3] - pointC[3]}
	local norm	= {
		-(ab[2] * bc[3] + (ab[3] * bc[2])),
		  ab[1] * bc[3] + (ab[3] * bc[1]),
		-(ab[1] * bc[2]) + (ab[2] * bc[1])}
	local dotProd	= norm[1] * light[1] + norm[2] * light[2] + norm[3] * light[3]
	local normMag 	= sqrt(norm[1]  * norm[1]  + norm[2]  * norm[2]  + norm[3]  * norm[3] )
	local lightMag	= sqrt(light[1] * light[1] + light[2] * light[2] + light[3] * light[3])
	return ( acos( dotProd / (normMag * lightMag ) ) / pi) * light.brightness
	end

function Ref3D.light.getNewColor( poly )
	light.factor = Ref3D.light.getLightFactor( poly )
	light.r = min( light.r * light.factor + light.ar, 255 )
	light.b = min( light.b * light.factor + light.ab, 255 )
	light.g = min( light.g * light.factor + light.ag, 255 )
	light.newColor = { light.r, light.g, light.b, 255 }
	return light.newColor
	end

function Ref3D.setColor( poly )
	if poly.color[1] then else poly.color={205,170,75} end
	light.r		= poly.color[1]
	light.g		= poly.color[2]
	light.b		= poly.color[3]
	gr.setColor( Ref3D.light.getNewColor( poly ) )
	end

return Ref3D
