-- main.lua for threadTest.love

d3	= require 'Ref3D'

function love.load()
	-------------------
	-- create thread --
	-------------------
	com = require 'Communicate'
	com.createMainThread()
	----------------------------
	-- set default parameters --
	----------------------------
	gr, lt, le		= love.graphics, love.timer, love.event
	pi, sin, cos	= math.pi, math.sin, math.cos
	wd, ht			= 400, 200
	gr.setMode( wd, ht)
	fontsize		= 14
	gr.setFont( gr.newFont( fontsize ) )
	gr.setCaption( " Press 's' to see communication traffic.")
	--------------------
	-- Ref3D settings --
	--------------------
	vp				= { x = wd/2, y = ht/2 }	-- vanishing point
	fl				= 500						-- focal length
	d3.load( fl, vp.x, vp.y )
	d3.light.load()
	light.brightness = .95
	light[1],light[2],light[3] = 10, 0, 100
	cullBackfacing = true
	gr.setBackgroundColor( 0, 5, 25, 255 )
	display_status	= false
	state			= 1
	deltaAngle		= 0.006
	-- love._openConsole()
	------------------
	-- get geometry --
	------------------
	points, polygons = d3.loadData( 'teapot.r3d' )
	end

function love.update(dt)
	if state			== 1 then
		success			= com.main.sendData( deltaAngle )
		if success then
			state		= 2
			deltaAngle		= 0.006
			end
		end
	if state			== 2 then
		local data		= com.main.getData()
		if data then
			points		= d3.extractPoints( data )
			state		= 1		-- if new points received, calculate next rotation (in thread)
			end
		end
	end

function love.draw()
	------------------------
	-- monitor frame rate --
	------------------------
	gr.setColor( 0, 155, 55, 255 )
	gr.print( 'fps: '..love.timer.getFPS( ),330 , 10 )
	--------------------
	-- draw 3D figure --
	--------------------
	for i = 1, #polygons do
		d3.light.setAmbience( 15, 15, 15 )
		d3.setColor( polygons[i] )
		tri = {points[polygons[i][1]],points[polygons[i][2]],points[polygons[i][3]]}
		mode = 'fill'
		d3.graphics.triangle( mode, cullBackfacing, tri, 80, 0, 1 )
		d3.graphics.triangle( mode, cullBackfacing, tri, -80, 0, 1 )
		end
	-----------------------------------------------------------
	-- option to monitor data transfer between main & thread --
	-----------------------------------------------------------
	if display_status then com.show_status() end

	end

function love.keypressed( key )
	if key == 'escape'	then love.event.push('quit') end
	if key == 's'		then display_status = not display_status end
	end


